* {{ 
  font-family: 'Lato' !important; 
}}

/*
QPushButton {{
  color: {QTMATERIAL_SECONDARYCOLOR};
  text-transform: none;
  background-color: {QTMATERIAL_PRIMARYCOLOR};
}}
*/

QLineEdit {{
  color: #ffffff; 
  border: 1px solid #5c5c5c;
}}
QTextEdit {{
  color: #ffffff; 
  border: 1px solid #5c5c5c;
}}

QTableView::item,
QTreeView::item,
QListView::item {{
  padding: 4px;
  min-height: 28px;
}}